package MColPro;

use warnings;
use strict;

=head1 NAME

MColPro - A simple monitoring and alarm system

=head1 VERSION

Version 1.10

=cut

our $VERSION = '1.12';

=head1 AUTHOR

WJIAN, C<< <cloveperl at gmail.com> >>

=head1 BUGS

Please report any bugs or feature requests to C<bug-mcolpro at rt.cpan.org>, or through
the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=MColPro>.  I will be notified, and then you'll
automatically be notified of progress on your bug as I make changes.




=head1 SUPPORT

You can find documentation for this module with the perldoc command.

    perldoc MColPro


You can also look for information at:

=over 4

=item * RT: CPAN's request tracker

L<http://rt.cpan.org/NoAuth/Bugs.html?Dist=MColPro>

=item * AnnoCPAN: Annotated CPAN documentation

L<http://annocpan.org/dist/MColPro>

=item * CPAN Ratings

L<http://cpanratings.perl.org/d/MColPro>

=item * Search CPAN

L<http://search.cpan.org/dist/MColPro/>

=back


=head1 ACKNOWLEDGEMENTS


=head1 LICENSE AND COPYRIGHT

Copyright 2013 WJIAN.

This program is free software; you can redistribute it and/or modify it
under the terms of either: the GNU General Public License as published
by the Free Software Foundation; or the Artistic License.

See http://dev.perl.org/licenses/ for more information.


=cut

1; # End of MColPro
